@@ -64,6 +64,7 @@
 //! ```
 
 use std::sync::{Arc, atomic::AtomicBool};
+use findshlibs::{IterationControl, SharedLibrary};
 
 #[doc(hidden)]
 pub use concat_idents::concat_idents as __concat_idents;
@@ -106,21 +107,15 @@ impl<T> CrossDylib<T> {
 	#[doc(hidden)]
 	#[inline]
 	#[cfg(not(debug_assertions))]
-	pub fn new_ref(&self) -> Arc<T> {
-		match unsafe { &*self.inner.get() } {
-			Some(ref inner) => inner.clone(),
-			None => unreachable!()
-		}
+	pub fn new_ref(&self) -> Option<Arc<T>> {
+		unsafe { &*self.inner.get() }.as_ref().map(|arc| arc.clone())
 	}
 
 	#[doc(hidden)]
 	#[inline]
 	#[cfg(debug_assertions)]
-	pub fn new_ref(&self) -> Arc<T> {
-		match &*self.inner.borrow() {
-			Some(inner) => inner.clone(),
-			None => unreachable!()
-		}
+	pub fn new_ref(&self) -> Option<Arc<T>> {
+		self.inner.borrow().as_ref().map(|arc| arc.clone())
 	}
 
 	/// Synchronizes the value across all loaded shared libraries. You must call this function before using a CrossDylib.
@@ -131,8 +126,6 @@ impl<T> CrossDylib<T> {
 	///
 	/// This function will panic in debug mode if a race condition occurs.
 	pub unsafe fn sync(&self) -> Result<(), libloading::Error> {
-		use findshlibs::{IterationControl, SharedLibrary};
-
 		assert!(self.symbol.len() > 0 && self.symbol.ends_with(&[0u8]));
 
 		let mut result = Ok(());
@@ -145,8 +138,17 @@ impl<T> CrossDylib<T> {
 
 		self.syncing.store(true, std::sync::atomic::Ordering::Release);
 
+		let process = std::env::current_exe().ok();
+
 		// Try and find a module that has already created this
 		findshlibs::TargetSharedLibrary::each(|shlib| {
+			if let Some(ref process) = process {
+				if shlib.name() == process {
+					// Skip the current process
+					return IterationControl::Continue;
+				}
+			}
+
 			let lib = match libloading::Library::new(shlib.name()) {
 				Ok(lib) => lib,
 				Err(err) => {
@@ -166,7 +168,7 @@ impl<T> CrossDylib<T> {
 						*inner = Some(init);
 						IterationControl::Break
 					} else {
-						// We just iterated over ourself
+						// We just iterated over ourself, or this module isn't ready yet
 						IterationControl::Continue
 					}
 				}
@@ -275,8 +277,9 @@ impl<T> std::ops::Deref for CrossDylib<T> {
 /// }
 /// ```
 macro_rules! crossdylib {
-	{ $($vis:vis static $ident:ident: $ty:ty = $expr:expr;)+ } => {
+	{ $($(#[$attr:meta])* $vis:vis static $ident:ident: $ty:ty = $expr:expr;)+ } => {
 		$(
+			$(#[$attr])*
 			$vis static $ident: $crate::CrossDylib<$ty> = $crate::CrossDylib {
 				inner: $crate::__new_container::<$ty>(),
 				symbol: concat!("__crossdylib_", stringify!($ident), "\0").as_bytes(),
@@ -292,10 +295,48 @@ macro_rules! crossdylib {
 					if $ident.syncing.load(std::sync::atomic::Ordering::Acquire) {
 						None
 					} else {
-						Some($ident.new_ref())
+						$ident.new_ref()
 					}
 				}
 			});
 		)+
 	};
+}
+
+/// Scans all loaded shared libraries for the specified procedure.
+pub unsafe fn scan_fn<T: Copy>(symbol: &[u8]) -> Result<Option<T>, libloading::Error> {
+	let mut result = Ok(None);
+
+	let process = std::env::current_exe().ok();
+
+	findshlibs::TargetSharedLibrary::each(|shlib| {
+		if let Some(ref process) = process {
+			if shlib.name() == process {
+				// Skip the current process
+				return IterationControl::Continue;
+			}
+		}
+
+		let lib = match libloading::Library::new(shlib.name()) {
+			Ok(lib) => lib,
+			Err(err) => {
+				result = Err(err);
+				return IterationControl::Break;
+			},
+		};
+
+		match lib.get::<T>(symbol) {
+			Err(libloading::Error::DlSym { .. }) | Err(libloading::Error::DlSymUnknown) | Err(libloading::Error::GetProcAddress { .. }) | Err(libloading::Error::GetProcAddressUnknown) => IterationControl::Continue,
+			Err(err) => {
+				result = Err(err);
+				IterationControl::Break
+			},
+			Ok(sym) => {
+				result = Ok(Some(*sym));
+				IterationControl::Break
+			}
+		}
+	});
+
+	result
 }
\ No newline at end of file
